# frozen_string_literal: true

require 'distributed_press/v1/social/client'

require_relative 'jekyll/activity_pub'
require_relative 'jekyll/activity_pub/notifier'

# Generate Actor and WebFinger pages
Jekyll::Hooks.register :site, :post_read, priority: :low do |site|
  Jekyll::ActivityPub::Notifier.site = site

  # Private key is mandatory during build, so any errors should stop the
  # build instead of logging a warning.
  begin
    private_key = File.read(site.config['activity_pub_private_key'])
  rescue StandardError
    Jekyll.logger.warn 'ActivityPub:', 'There\'s an issue with your private key'

    raise
  end

  # We only need the public key at this point
  client = DistributedPress::V1::Social::Client.new(public_key_url: nil, private_key_pem: private_key)

  site.pages << site.config['actor'] = actor = Jekyll::ActivityPub::Actor.new(site)
  site.pages << site.config['public_key'] = Jekyll::ActivityPub::PublicKey.new(site, actor, client.public_key)
  site.pages << site.config['webfinger'] = webfinger = Jekyll::ActivityPub::WebFinger.new(site, actor)
  site.pages << site.config['outbox'] = Jekyll::ActivityPub::Outbox.new(site, actor)
  site.pages << site.config['host-meta'] = Jekyll::ActivityPub::HostMeta.new(site, webfinger)
  site.pages << Jekyll::ActivityPub::Following.new(site, actor)
  site.pages << site.config['followers'] = Jekyll::ActivityPub::Followers.new(site, actor)

  # Add Actor to home page
  home = site.pages.find do |page|
    page.url == '/'
  end

  if home
    Jekyll.logger.info 'ActivityPub:', 'Adding Actor to home page'
    home.data['activity'] = actor
  end
end

# Generate an activity for each document after the content has been
# rendered as HTML.
Jekyll::Hooks.register(:documents, :post_convert, priority: :high) do |doc|
  next unless doc.write?
  next if doc.data['sitemap'] == false
  next if doc.data['activity'] == false

  site = doc.site
  dir = File.dirname(doc.url)
  name = "#{File.basename(doc.url, '.html')}.jsonld"
  actor = site.config['actor']
  outbox = site.config['outbox']
  followers = site.config['followers']

  Jekyll::ActivityPub::Activity.new(site, actor, doc, nil, dir, name).tap do |activity|
    doc.data['activity'] = activity

    created_at = doc.data['created_at'] || doc.date
    updated_at = doc.data['last_modified_at'] || doc.date

    create_or_update =
      if updated_at > created_at
        Jekyll::ActivityPub::Update
      else
        Jekyll::ActivityPub::Create
      end

    create_or_update.new(site, actor, followers, activity).tap do |action|
      Jekyll::ActivityPub::Notifier.public_send action.data['type'].downcase.to_sym, activity.destination(site.dest)

      outbox.data['totalItems'] += 1
      outbox.data['orderedItems'] << action
    end

    site.pages << activity
  end
end

# This hook is shit because we don't have access to the site!
Jekyll::Hooks.register(:clean, :on_obsolete, priority: :low) do |list|
  list.each do |path|
    Jekyll::ActivityPub::Notifier.delete path
  end
end

# Store data generated between builds and commits if
# jekyll-write-and-commit-changes is enabled.
Jekyll::Hooks.register(:site, :post_write, priority: :low) do |_|
  Jekyll::ActivityPub::Notifier.save
end
