# frozen_string_literal: true

require 'jekyll/page'
require_relative 'helper'
require_relative 'image'
require_relative 'property_value'
require_relative 'errors'
require_relative 'notifier'

module Jekyll
  module ActivityPub
    # Points to site's author profile
    class Actor < Jekyll::Page
      include Helper

      # Initialize with default data
      #
      # @param :site [Jekyll::Site]
      # @param :base [String]
      # @param :dir [String]
      # @param :name [String]
      def initialize(site, base = '', dir = '', name = 'about.jsonld')
        @context = StubContext.new(registers: { site: site })
        super

        Notifier.actor_url = data['id']

        trigger_hooks :post_init
      end

      def read_yaml(*)
        self.data = {
          '@context' => [
            'https://www.w3.org/ns/activitystreams',
            {
              '@language' => locale,
              'schema' => 'http://schema.org#',
              'PropertyValue' => 'schema:PropertyValue',
              'value' => 'schema:value'
            }
          ],
          'type' => 'Person',
          'id' => absolute_url(url),
          'outbox' => nil,
          'inbox' => inbox,
          'following' => nil,
          'followers' => nil,
          'preferredUsername' => username,
          'name' => public_name,
          'summary' => summary,
          'icon' => icons,
          'publicKey' => nil,
          'attachment' => [
            PropertyValue.new(website_name, absolute_url(site.config['url']))
          ]
        }
      end

      private

      # Finds public name
      #
      # @return [String,nil]
      def public_name
        @public_name ||= find_best_value_for(site.config, %w[activity_pub public_name], 'title') || username
      end

      # Find summary between site tagline or description, reusing fields
      # that are used on jekyll-seo-tag
      #
      # @return [String,nil]
      def summary
        @summary ||= find_best_value_for(site.config, %w[activity_pub summary], 'tagline', 'description')
      end

      # Find icons
      #
      # @return [Array]
      def icons
        @icons ||= [find_best_value_for(site.config, %w[activity_pub icons], 'logo')].flatten.compact.map do |icon|
          Image.new(site, icon, summary)
        end
      end

      # Finds the website name
      #
      # @return [String]
      def website_name
        @website_name ||= find_best_value_for(site.config, %w[activity_pub website name]) || 'Website'
      end

      def strict?
        true
      end

      def inbox
        @inbox ||=
          site.config.dig('activity_pub', 'url').tap do |inbox|
            value_is_required! inbox, MissingInboxError

            inbox << '/v1/@'
            inbox << username
            inbox << '@'
            inbox << hostname
          end
      end
    end
  end
end
