# frozen_string_literal: true

require 'pathname'
require 'httparty'
require 'distributed_press/v1/social/client'
require_relative 'errors'
require_relative 'create'
require_relative 'update'
require_relative 'delete'

module Jekyll
  module ActivityPub
    # Long term store for notifications.
    #
    # Needs to be a singleton so we can use the same data across all of
    # Jekyll's build process.
    class Notifier
      # An struct that responds to a #url method
      PseudoObject = Struct.new(:url, keyword_init: true)

      class << self
        # Set the site and initialize data
        #
        # @param :site [Jekyll::Site]
        # @return [Jekyll::Site]
        def site=(site)
          @@site = site.tap do |s|
            s.data['activity_pub'] ||= {}
            s.data['activity_pub']['notifications'] ||= {}
          end
        end

        # Save the public key URL for later
        #
        # @param :url [String]
        def public_key_url=(url)
          data['public_key_url'] = url
        end

        # Public key URL
        #
        # @return [String,nil]
        def public_key_url
          data['public_key_url']
        end

        def actor=(actor)
          data['actor'] = actor
        end

        def actor
          data['actor']
        end

        def actor_url=(url)
          data['actor_url'] = url
        end

        def actor_url
          data['actor_url']
        end

        # Send notifications
        #
        # 1. Wait for public key propagation
        # 2. Create/update inbox
        # 3. Send create, update, and delete
        def notify!
          # TODO: request several times with a timeout
          response = HTTParty.get(public_key_url)

          unless response.ok?
            raise NotificationError,
                  "Could't fetch public key (#{response.code}: #{response.message})"
          end

          unless client.private_key.compare? OpenSSL::Pkey::RSA.new(response.body)
            raise NotificationError, "Public key at #{public_key_url} differs from local version"
          end

          base_endpoint = "/v1/#{actor}/"
          outbox_endpoint = "#{base_endpoint}/outbox/"
          actor_object = PseudoObject.new(url: actor_url)

          # Create inbox
          # TODO: Send keypair
          client.post(endpoint: base_endpoint, body: {})

          # Remove notifications already performed and notify
          data['notifications'].reject do |object_url, status|
            done? object_url, status
          end.each do |object_url, status|
            process_object(outbox_endpoint, actor_object, PseudoObject.new(url: object_url), status)
          end

          # Store everything for later
          save
        end

        # @return [Jekyll::Site]
        def site
          @@site
        end

        # Return data
        #
        # @return [Hash]
        def data
          @@data ||= site.data['activity_pub']
        end

        # Removes an activity if it was previously created
        #
        # @param :path [String]
        # @return [Hash]
        def delete(path)
          action(path, 'delete') if exist? path
        end

        # Updates an activity if it was previously created
        #
        # @param :path [String]
        # @return [Hash]
        def update(path)
          action(path, 'update') if exist? path
        end

        # Creates an activity
        #
        # @param :path [String]
        # @return [Hash]
        def create(path)
          action(path, 'create')
        end

        # Check if activity existed
        #
        # @param :path [String]
        # @return [Boolean]
        def exist?(path)
          data['notifications'].key? path_relative_to_dest(path)
        end

        # Stores data back to a file and optionally commits it
        #
        # @return [nil]
        def save
          # TODO: Send warning if CI is detected
          Jekyll.logger.info 'ActivityPub:', "Saving data to #{relative_path}"

          FileUtils.mkdir_p(File.dirname(path))

          File.open(path, 'w') do |f|
            f.flock(File::LOCK_EX)
            f.rewind
            f.write(YAML.dump(data))
            f.flush
            f.truncate(f.pos)
          end

          if ENV['JEKYLL_ENV'] == 'production' && site.respond_to?(:repository)
            site.staged_files << relative_path
            site.repository.commit 'ActivityPub'
          end

          nil
        end

        # Returns the path for the storage
        #
        # @return [String]
        def path
          @@path ||= site.in_source_dir(site.config['data_dir'], 'activity_pub.yml')
        end

        # Storage path relative to site source
        #
        # @return [String]
        def relative_path
          @@relative_path ||= Pathname.new(path).relative_path_from(site.source).to_s
        end

        private

        # Finds the private key path on config
        #
        # @return [String, nil]
        def private_key_path
          @@private_key_path ||= site.config['activity_pub_private_key']
        end

        # Returns the private key
        #
        # @return [String, nil]
        def private_key
          @@private_key ||= File.read private_key_path
        rescue StandardError
          Jekyll.logger.warn 'ActivityPub:', 'There\'s an issue with your private key'
          raise
        end

        # @return [Hash]
        def config
          @@config ||= site.config['activity_pub'] || {}
        end

        def client
          @@client ||= DistributedPress::V1::Social::Client.new(
            private_key_pem: private_key,
            url: config['url'],
            public_key_url: public_key_url
          )
        end

        # Run action
        #
        # @param :path [String]
        # @param :action [String]
        # @return [Hash]
        def action(path, action)
          path = path_relative_to_dest(path)

          data['notifications'][path] ||= {}
          data['notifications'][path]['action'] = action.to_s
        end

        # Paths are relative to site destination
        #
        # @param :path [String]
        # @return [String]
        def path_relative_to_dest(path)
          Pathname.new(site.in_dest_dir(path)).relative_path_from(site.dest).to_s
        end

        # Detects if an action was already done
        def done?(url, status)
          (status['action'] == 'done').tap do |done|
            Jekyll.logger.debug('ActivityPub:', "Skipping notification for #{url}") if done
          end
        end

        # Turns an object into an activity and notifies outbox
        #
        # @param :endpoint [String]
        # @param :actor [PseudoObject]
        # @param :object [PseudoObject]
        # @param :status [Hash]
        # @return [nil]
        def process_object(endpoint, actor, object, status)
          action = status['action']
          activity = Object.const_get(action.capitalize).new(site, actor, object)

          if (response = client.post(endpoint: endpoint, body: activity)).ok?
            status['action'] = 'done'
            status["#{action}d_at"] = Time.now.to_i
          else
            Jekyll.logger.warn 'ActivityPub:',
                               "Couldn't perform #{action} for #{object_url} (#{response.code}: #{response.message})"
          end

          nil
        rescue NameError
          Jekyll.logger.warn 'ActivityPub:', "Action \"#{action}\" for #{url} unrecognized, ignoring."
        end
      end
    end
  end
end
